//
//  ItemCollectionParty.swift
//  ArkitDoushiQi
//
//  Created by Louis DUFOUR on 31/05/2024.
//

import SwiftUI

struct ItemCollectionParty: View {
    var party: Party
    
    var body: some View {
        VStack {
            HStack {
                Text(party.date)
                    .font(.headline)
                    .foregroundColor(.gray)
                Spacer()
            }
            .padding(.bottom, 5)
            
            GeometryReader { geometry in
                HStack(spacing: 10) {
                    HStack {
                        Image(party.player1Image)
                            .resizable()
                            .aspectRatio(contentMode: .fill)
                            .frame(width: geometry.size.width * 0.15, height: geometry.size.width * 0.15)
                            .clipShape(Circle())
                            .overlay(Circle().stroke(Color.purple, lineWidth: 2))
                        
                        VStack(alignment: .leading) {
                            Text(party.player1Name)
                                .fontWeight(.bold)
                                .foregroundColor(.purple)
                                .lineLimit(1)
                                .minimumScaleFactor(0.5)
                                .frame(maxWidth: geometry.size.width * 0.25, alignment: .leading)
                            
                            Text("\(party.player1Score)")
                                .font(.largeTitle)
                                .fontWeight(.bold)
                                .foregroundColor(.green) // Gagnant en vert
                        }
                    }
                    
                    Text("vs")
                        .font(.headline)
                        .foregroundColor(.gray)
                    
                    HStack {
                        VStack(alignment: .trailing) {
                            Text(party.player2Name)
                                .fontWeight(.bold)
                                .foregroundColor(.purple)
                                .lineLimit(1)
                                .minimumScaleFactor(0.5)
                                .frame(maxWidth: geometry.size.width * 0.25, alignment: .trailing)
                            
                            Text("\(party.player2Score)")
                                .font(.largeTitle)
                                .fontWeight(.bold)
                                .foregroundColor(.red) // Perdant en rouge
                        }
                        
                        Image(party.player2Image)
                            .resizable()
                            .aspectRatio(contentMode: .fill)
                            .frame(width: geometry.size.width * 0.15, height: geometry.size.width * 0.15)
                            .clipShape(Circle())
                            .overlay(Circle().stroke(Color.purple, lineWidth: 2))
                    }
                }
                .padding()
                .background(Color.white)
                .cornerRadius(15)
                .shadow(radius: 3) // Réduction de l'ombre pour un effet plus léger
            }
            .frame(height: 120) // Ajustez la hauteur en fonction de votre contenu
        }
        .padding(.horizontal)
    }
}

struct ItemCollectionParty_Previews: PreviewProvider {
    static var previews: some View {
        ItemCollectionParty(party: Party(player1Name: "L'invaincu du samedi", player1Score: 2, player1Image: "Perceval", player2Name: "Le gars du dimanche", player2Score: 1, player2Image: "Perceval", date: "Samedi soir"))
            .previewLayout(.sizeThatFits)
            .padding()
    }
}
